#
# display.nawk
#
# Limitations: Targets in the range of 0-999
#              Paths in the range of 0-999
#
# Execution: ldinfo -r -v -S -t {target#} | \
#            nawk -f display.nawk current_device_name={device name} 
#
# Created 5/24/2000 -- cww

BEGIN { FS = "="; npaths = 0 }

{
   if ( $1 ~ /^ targ[0-9][0-9]?[0-9]?_path[0-9].*$/ )
   {
      if ( substr($1, 8, 1) ~ /[0-9]/ )
         ctarg = substr($1, 6, 3)
      else if ( substr($1, 7, 1) ~ /[0-9]/ )
         ctarg = substr($1, 6, 2)
      else
         ctarg = substr($1, 6, 1)

      if ( substr($1, 13 + length(ctarg), 1) ~ /[0-9]/ )
         cpath = substr($1, 11 + length(ctarg), 3)
      else if ( substr($1, 12 + length(ctarg), 1) ~ /[0-9]/ )
         cpath = substr($1, 11 + length(ctarg), 2)
      else
         cpath = substr($1, 11 + length(ctarg), 1)
   }
   if ( $1 ~ /^targ[0-9][0-9]?[0-9]?_nPaths.*$/ )
   {
      npaths = $2
   }
   else if ( $1 ~ /^ targ[0-9][0-9]?[0-9]?_path0_array.*$/ ) 
   {
      storage_system_wwn = substr($2, 2, length($2) - 2) 
   }
   else if ( $1 ~ /^ targ[0-9][0-9]?[0-9]?_path0_device.*$/ )
   {
      lun_wwn = substr($2, 2, length($2) - 2)
   }
   else if ( $1 ~ /^ targ[0-9][0-9]?[0-9]?_path0_tar.*$/ )
   {
      online_tar = $2 
   }
   else if ( $1 ~ /^ targ[0-9][0-9]?[0-9]?_path0_lun.*$/ )
   {
      if( raid_system == "HSZ" && online_tar != "0" )
      {
         online_lun = sprintf("D%s%02s", online_tar, $2)
      }
      else
      {
         online_lun = sprintf("D%s", $2)
      }	
   }
   else if ( $1 ~ /^ targ[0-9][0-9]?[0-9]?_path[0-9][0-9]?[0-9]?_ctlr.*$/ )
   {
      path_sn[cpath] = substr($2, 2, length($2) - 2)
   }
   else if ( $1 ~ /^ targ[0-9][0-9]?[0-9]?_path[0-9][0-9]?[0-9]?_preferred.*$/ )
   {
      path_preferred[cpath] = $2
   }
   else if ( $1 ~ /^ targ[0-9][0-9]?[0-9]?_path[0-9][0-9]?[0-9]?_state.*$/ )
   {
      path_state[cpath] = substr($2, 2, length($2) - 2)
   }
   else if ( $1 ~ /^ targ[0-9][0-9]?[0-9]?_path[0-9][0-9]?[0-9]?_parent.*$/ )
   {
      path_parent[cpath] = substr($2, 2, length($2) - 2)
   }
   else if ( $1 ~ /^ targ[0-9][0-9]?[0-9]?_path[0-9][0-9]?[0-9]?_self.*$/ )
   {
      path_self[cpath] = substr($2, 2, length($2) - 2)
      path_inst[cpath] = substr(path_self[cpath], 4)
      parent_path[cpath] = " "
      FS = " "
      while ( (getline < "/etc/path_to_inst") > 0 ) 
      {
         if ( $2 == path_inst[cpath] && $1 ~ /mda/ )
         {
            parent_path[cpath] = substr($1, 2, index($1, "mda") - 3)
         }
      }
      FS = "="
      close("/etc/path_to_inst")
      if( parent_path[cpath] ~ /fibre-channel/ || parent_path[cpath] ~ /fca/ )
      {
         raid_system="HSG"
      }
      else  
      {
         raid_system="HSZ"
      }
   }
}

END { 
       preferred_path = "None" 
       subsystem_state = "Dead"
       for ( i = 0 ; i < npaths; i++ )
       {
          if ( path_preferred[i] == "1" ) 
             preferred_path = path_sn[i]
          if ( path_state[i] == "current" )
          {
             subsystem_state = "Operational"
             path_state[i] = "online"
          }
          if ( path_sn[i] == "" )
             online_luns[i] = " "
          else
             online_luns[i] = online_lun
       }
       if ( subsystem_state == "Dead" )
       {
          for( i = 0 ; i < npaths; i++ )
             online_luns[i] = " "
       }
       print ""
       print ""
       printf "Device: %s     Status: %s\n", current_device_name, subsystem_state
       print ""
       printf "Storage System: %s\n", storage_system_wwn
       printf "LUN: %s\n", lun_wwn
       printf "Preferred Path: %s\n", preferred_path
       print "=============================================================="
       print "Controller   Unit   State      HBA        Path"
       print "=============================================================="
       for ( i = 0 ; i < npaths ; i++ )
          printf "%-12s %-6s %-10s %-10s %s\n", path_sn[i], online_luns[i], path_state[i], path_parent[i], parent_path[i]
}  
       
